@@ -5,7 +5,7 @@ import os
 import pathlib
 import tempfile
 
-from borgmatic.borg import feature, state
+from borgmatic.borg import environment, feature, flags, state
 from borgmatic.execute import DO_NOT_CAPTURE, execute_command, execute_command_with_processes
 
 logger = logging.getLogger(__name__)
@@ -98,16 +98,19 @@ def deduplicate_directories(directory_devices):
     return tuple(sorted(deduplicated))
 
 
-def write_pattern_file(patterns=None):
+def write_pattern_file(patterns=None, sources=None):
     '''
-    Given a sequence of patterns, write them to a named temporary file and return it. Return None
-    if no patterns are provided.
+    Given a sequence of patterns and an optional sequence of source directories, write them to a
+    named temporary file (with the source directories as additional roots) and return the file.
+    Return None if no patterns are provided.
     '''
     if not patterns:
         return None
 
     pattern_file = tempfile.NamedTemporaryFile('w')
-    pattern_file.write('\n'.join(patterns))
+    pattern_file.write(
+        '\n'.join(tuple(patterns) + tuple(f'R {source}' for source in (sources or [])))
+    )
     pattern_file.flush()
 
     return pattern_file
@@ -203,7 +206,7 @@ def create_archive(
     progress=False,
     stats=False,
     json=False,
-    files=False,
+    list_files=False,
     stream_processes=None,
 ):
     '''
@@ -216,7 +219,7 @@ def create_archive(
     sources = deduplicate_directories(
         map_directories_to_devices(
             expand_directories(
-                location_config['source_directories']
+                location_config.get('source_directories', [])
                 + borgmatic_source_directories(location_config.get('borgmatic_source_directory'))
             )
         )
@@ -226,14 +229,14 @@ def create_archive(
         working_directory = os.path.expanduser(location_config.get('working_directory'))
     except TypeError:
         working_directory = None
-    pattern_file = write_pattern_file(location_config.get('patterns'))
+    pattern_file = write_pattern_file(location_config.get('patterns'), sources)
     exclude_file = write_pattern_file(
         expand_home_directories(location_config.get('exclude_patterns'))
     )
     checkpoint_interval = storage_config.get('checkpoint_interval', None)
     chunker_params = storage_config.get('chunker_params', None)
     compression = storage_config.get('compression', None)
-    remote_rate_limit = storage_config.get('remote_rate_limit', None)
+    upload_rate_limit = storage_config.get('upload_rate_limit', None)
     umask = storage_config.get('umask', None)
     lock_wait = storage_config.get('lock_wait', None)
     files_cache = location_config.get('files_cache')
@@ -246,22 +249,22 @@ def create_archive(
         atime_flags = ('--noatime',) if location_config.get('atime') is False else ()
 
     if feature.available(feature.Feature.NOFLAGS, local_borg_version):
-        noflags_flags = ('--noflags',) if location_config.get('bsd_flags') is False else ()
+        noflags_flags = ('--noflags',) if location_config.get('flags') is False else ()
     else:
-        noflags_flags = ('--nobsdflags',) if location_config.get('bsd_flags') is False else ()
+        noflags_flags = ('--nobsdflags',) if location_config.get('flags') is False else ()
 
     if feature.available(feature.Feature.NUMERIC_IDS, local_borg_version):
-        numeric_ids_flags = ('--numeric-ids',) if location_config.get('numeric_owner') else ()
+        numeric_ids_flags = ('--numeric-ids',) if location_config.get('numeric_ids') else ()
     else:
-        numeric_ids_flags = ('--numeric-owner',) if location_config.get('numeric_owner') else ()
+        numeric_ids_flags = ('--numeric-owner',) if location_config.get('numeric_ids') else ()
 
     if feature.available(feature.Feature.UPLOAD_RATELIMIT, local_borg_version):
         upload_ratelimit_flags = (
-            ('--upload-ratelimit', str(remote_rate_limit)) if remote_rate_limit else ()
+            ('--upload-ratelimit', str(upload_rate_limit)) if upload_rate_limit else ()
         )
     else:
         upload_ratelimit_flags = (
-            ('--remote-ratelimit', str(remote_rate_limit)) if remote_rate_limit else ()
+            ('--remote-ratelimit', str(upload_rate_limit)) if upload_rate_limit else ()
         )
 
     ensure_files_readable(location_config.get('patterns_from'), location_config.get('exclude_from'))
@@ -290,7 +293,7 @@ def create_archive(
         + (('--remote-path', remote_path) if remote_path else ())
         + (('--umask', str(umask)) if umask else ())
         + (('--lock-wait', str(lock_wait)) if lock_wait else ())
-        + (('--list', '--filter', 'AME-') if files and not json and not progress else ())
+        + (('--list', '--filter', 'AME-') if list_files and not json and not progress else ())
         + (('--info',) if logger.getEffectiveLevel() == logging.INFO and not json else ())
         + (('--stats',) if stats and not json and not dry_run else ())
         + (('--debug', '--show-rc') if logger.isEnabledFor(logging.DEBUG) and not json else ())
@@ -298,17 +301,13 @@ def create_archive(
         + (('--progress',) if progress else ())
         + (('--json',) if json else ())
         + (tuple(extra_borg_options.split(' ')) if extra_borg_options else ())
-        + (
-            '{repository}::{archive_name_format}'.format(
-                repository=repository, archive_name_format=archive_name_format
-            ),
-        )
-        + sources
+        + flags.make_repository_archive_flags(repository, archive_name_format, local_borg_version)
+        + (sources if not pattern_file else ())
     )
 
     if json:
         output_log_level = None
-    elif (stats or files) and logger.getEffectiveLevel() == logging.WARNING:
+    elif (stats or list_files) and logger.getEffectiveLevel() == logging.WARNING:
         output_log_level = logging.WARNING
     else:
         output_log_level = logging.INFO
@@ -317,6 +316,8 @@ def create_archive(
     # the terminal directly.
     output_file = DO_NOT_CAPTURE if progress else None
 
+    borg_environment = environment.make_environment(storage_config)
+
     if stream_processes:
         return execute_command_with_processes(
             full_command,
@@ -325,6 +326,7 @@ def create_archive(
             output_file,
             borg_local_path=local_path,
             working_directory=working_directory,
+            extra_environment=borg_environment,
         )
 
     return execute_command(
@@ -333,4 +335,5 @@ def create_archive(
         output_file,
         borg_local_path=local_path,
         working_directory=working_directory,
+        extra_environment=borg_environment,
     )
